#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include "globals.h"

void consume(int id, int fd);

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        fprintf(stderr, "Usage: id\n");
        return -1;
    }

    int id = atoi(argv[1]);

    // First open the FIFO in read-only mode
    // and make it nonblocking.
    int fd = open(FIFO_PATH, O_RDONLY);
    fcntl(fd, F_SETFL, O_NONBLOCK);

    // Consume items from the FIFO.
    consume(id, fd);

    return 0;
}

void consume(int id, int fd)
{
    char buffer[BUFFER_SIZE];
    char item[BUFFER_SIZE];
    bool eof = false;

    do
    {
        ssize_t size = read(fd, buffer, BUFFER_SIZE);

        switch (size)
        {
            case -1:
            {
                if (errno == EAGAIN)
                {
                    // Pipe empty. Snooze and try again.
                    printf("(empty)\n");
                    sleep(1);
                    break;
                }
                else exit(-id);
            }

            case 0:
            {
                // End of file.
                eof = true;
                break;
            }

            default:
            {
                printf("(read %d)\n", (int)size);

                int i = 0;
                do
                {
                    strcpy(item, buffer + i);
                    i += strlen(item) + 1;

                    printf("Consumer %d read %s", id, item);
                    fflush(stdout);
                } while (i < size);

                break;
            }
        }
    } while (!eof);

    printf("\nConsumer %d done!\n", id);
}
